C23456789.123456789.123456789.123456789.123456789.123456789.123456789.12
      subroutine atscat_diff_calc_norm ( n_ein,n_ener,nlat,ntheta,nphi,
     .  at_scat_data,e_in,lat_edge,
     .  lat_cent,theta_cent,phi_cent,use_coslat_corr, theta_geo,phi_geo,
     .  theta_source,phi_source,detector,drmdbpath,list,lptr,lend,x,y,z,
     .  azimuth, zenith, milliaz, millizen, leaf_ver, n, maxen,maxchan, 
     .  debug,energ_lo,energ_hi,edif_edge_lo, edif_edge_hi, ienerg, 
     .  ichan, ebin_edge_out, nobins_out, ebin_edge_in, nobins_in, 
     .  new_atscat_matrix)
C Changes 11-05-08 
C 1.Geoff's energy interpolation code for the scattered database has been
C  removed. Now the scattered database is pre-processed so that its input and
C  output energies match the input energies of Marc's DRM database. Then the 
C  appropriate scattered matrix is simply matrix multiplied with the direct 
C  matrix for that direction, after the direct matrix has been energy
C  interpolated (echan_integrator.f) to match the user desired channel edges on
C  the output side. The convolved matrices for each direction are summed to 
C  form a single atmospheric scattering matrix. This matrix is then interpolated
C  over the input side (atscat_highres_ephoton_interpolator.f) to match the user
C  desired photon (input) energies.
C 2. abs(cos(lat)) correction is now applied inside this code instead of 
C  to the database so that the correction is for the actual latitude instead 
C  of the bin center.
C 3. Interpolation over latitude has been moved inside this code and the
C subroutine samp_calc is no longer used.
C Changes 11-10-2008 the input variable leaf_ver has been added. This is used
C  by parse_drm_filename inside sky_interp_lite and refers to the content
C  version number for the leaf drm files.
C
C Test version - input the direct matrix in units of cm^2 (don't divide by 
C energy)      
C Inputs
C n_ein = number of incident scattering energies (keV)
C n_ener = number of output scattering energies for each input energy (keV)
C nlat = number of latitude bins
C ntheta = number of polar angle bins coming up from the atmosphere toward the
C          spacecrart
C nphi = number of azimuthal scattering angles 
C n_ener_abin
C n_ener_low
C at_scat_data - atmospheric scattering database
C e_in - incident scattering energies (keV)
C lat_edge - latitude bin edges (deg)
C theta_cent - centers of polar angle bins coming up from atmosphere toward s/c
C phi_cent - azimuthal bin edges
C use_coslat_corr ne 0: apply the cosine(latitude) correction
C use_coslat_corr = 0: turn off cosine(latitude) correction factor.
C theta_geo, phi_geo - zenith and azimuth angles for geocenter in s/c coords
C theta_source, phi_source - zenith and azimuth angles for source in s/c coords
C detector = the detector number of interest (NaI detectors 0-11, BGO 12-13)
C The following inputs are needed for the call to sky_interp
C drmdbpath = the path to the grid of response matrices
C LIST,LPTR, LEND - triangulation indices needed by Renka's TRFIND routine.
C X,Y,Z - cartesian coordinates (S/C) for each grid point. Needed for Renka's
C          TRFIND routine.
C Azimuth, Zenith - S/C coordinates in degrees for each grid point. Needed to
C                   find the corresponding response file.
C milliaz, millizen - S/C coordinates in millidegress for each grid point. Used
C by parse_drm_filename inside sky_interp_lite
C leaf_ver - version number of leaf drm files. Used by parse_drm_filename inside
C  sky_interp_lite
C N - the number of points in the triangulation
C maxen = the maximum number of photon energies included in the output matrix
C maxchan = the maximum number of channel energies included in the output matrix
C debug >1 increases verbosity
C energ_lo, energ_hi = the photon energy boundaries for the output matrix
C echan_min, echan_max = the channel energy boundaries for the output matrix
C atscat_diff_matrix = the output atmospheric scattering differential matrix 
C  in units of cm^2/keV for the sky position theta_source,phi_source, and the
C  geocenter position theta_geo, phi_geo
C ichan, ienerg = the actual number of channel energies, photon energies
C                 included in the output matrix.

      implicit none
      integer n_ein, n_ener, nlat, nphi, ntheta
      real*4 at_scat_data(n_ein, n_ener, nlat, ntheta, nphi)
      real*4 e_in(n_ein),lat_edge(nlat+1),theta_cent(ntheta)
      real*4 lat_cent(nlat),phi_cent(nphi)
      real*4 theta_geo, phi_geo, theta_source, phi_source
      integer nobins_out, nobins_in
      real ebin_edge_out(nobins_out+1), ebin_edge_in(nobins_in+1)
C inputs to sky_interp
      real X(*),Y(*),Z(*),Azimuth(*),Zenith(*)
      integer detector,maxen,maxchan,list(*),lptr(*),lend(*),n,
     . debug, milliaz(*), millizen(*),use_coslat_corr
      character*3 leaf_ver
      character*80 drmdbpath
      real energ_lo(*), energ_hi(*),
     .     outmatrix(maxen,maxchan),edif_edge_lo(maxen,maxchan),
     .     edif_edge_hi(maxen,maxchan)
      integer ichan,ienerg
C Outputs
C atscat_diff_matrix - the differential atmospheric scattering matrix. This
C  can be passed directly to the energy integrator or added to the differential
C  direct matrix and then passed to the energy integrator.
      real*4 new_atscat_matrix(maxen,maxchan)
C local variables
      integer i,j,k, il_low, il_high, itheta, iphi, ips, ien, ivlow
      integer ie_in_l, ie_in_h, iv, ivfind, ivhigh, nvpoints, ie_out_l
      integer ie_out_h, ihselcnt, n_iuse, is2, intrp, ihlow, ihhigh
      integer ilow, ntheta_loopmax
      real*4 ein_atscatlow, ein_atscathigh, e_vert_edge(ienerg+1)
      real*4 e_vert_cent(ienerg),ein, ein_fract,e_cut
      real*4 gx(3), gy(3), gz(3), sl(3), lat
      real*4 last_dirx, last_diry, last_dirz
      real*4 theta_u, phi_u, dirx, diry, dirz,az, el
      real*4 radeg
      real*4 e_edge(n_ener+1), e_cent_scat(n_ener), e_wide_scat(n_ener)
      real*4 ecent_diff_matrix(ienerg,ichan) 
      real*4 direct_diff_matrix(maxen,maxchan),at_scat_samp, l_fract
      real*4 edge_horiz_diff_matrix(ienerg,ichan+1),drm_bit, fract
      real*4 at_scat_coeff, vlow, vhigh, vwide, astep, estepl,eout_fract
      character*40 drmfiles(3)
      real*4 atscat_diff_matrix(maxen,maxchan),coslat_corr
      radeg=180.0/acos(-1.0)
C open file to dump output in scratch area
c      open(12,file='/Users/Shared/scratch/drm_bit.dat',status='NEW')      
C print statements for debugging   
      if (debug.gt.0) then   
        print*,'n_ein,n_ener,nlat, ntheta,nphi:',
     .  n_ein, n_ener, nlat, ntheta, nphi
        print*,'e_in: ',(e_in(i),i=1,n_ein)
        print*,'lat_edge: ',(lat_edge(i),i=1,nlat+1)
        print*,'lat_cent: ',(lat_cent(i),i=1,nlat)
        print*,'theta_cent: ',(theta_cent(i),i=1,ntheta)
        print*,'phi_cent: ',(phi_cent(i),i=1,nphi)
        print*,'theta_source, phi_source: ',theta_source,phi_source
        print*,'theta_geo, phi_geo: ',theta_geo,phi_geo
        print*,'ienerg, ichan: ',ienerg,ichan
        print*,'at_scat_data(7,*,5,1,1): ',
     .   (i,at_scat_data(7,i,5,1,1),i=1,n_ener)
      endif	
C pre-loop calculation tasks
      last_dirx=0.0 
      last_diry=0.0
      last_dirz=0.0
      do i=1,ienerg
	do j=1,nobins_out
	  atscat_diff_matrix(i,j)=0.0
        enddo
      enddo
      
C calculate coordinate transforms      
      call geocords ( theta_geo, phi_geo, theta_source, phi_source,
     . gx,gy,gz,sl)
C calculate latitude  - changed from 180-cos to cos 6/25/08    
      lat=180-radeg*acos(sl(1)*gz(1)+sl(2)*gz(2)+sl(3)*gz(3))
      if (debug.gt.1) then
        print*,'lat = ',lat
        print*,'radeg,sl(1:3),gz(1:3)',radeg,(sl(i),i=1,3),(gz(i),i=1,3)
        print*,'gx(1:3),gy(1:3) :',(gx(i),i=1,3),(gy(i),i=1,3)
        print*,'Theta_geo, phi_geo, theta_source, phi_source: ',
     .  Theta_geo, phi_geo, theta_source, phi_source 
        print*,'lat_cent: ',(lat_cent(i),i=1,nlat)
        print*,'lat_edge: ',(lat_edge(i),i=1,nlat+1)
      endif
C compute cosine(latitude) correction factor if use_coslat_corr ne 0
      if (use_coslat_corr.ne.0) then 
        coslat_corr = abs(cos(lat/radeg))
      else
        coslat_corr = 1.0
      endif		      	
C put in lat_cut loop
      if (lat.le.lat_edge(nlat+1)) then
C calculate latitude interpoliation info: bounding bin indicies
C and weights
	if ((lat.gt.lat_cent(nlat)).or.(lat.ge.130)) then
	    print*,'lat = ',lat, ' > ',lat_cent(nlat)
	    print*,'Assuming scattering = 0.0 for all energies.'
	    goto 500
        endif

        if( lat .le. lat_cent(1) ) then 
          il_low=1
          il_high=2
          l_fract=0.0
        endif ! if( lat le lat_cent(1) )

        do i=1,nlat-1
          if((lat.gt.lat_cent(i)).and.(lat.le.lat_cent(i+1))) then 
            il_low=i
            il_high=i+1
            l_fract=1.0-(lat-lat_cent(i))/(lat_cent(i+1)-lat_cent(i))
	  endif ! if(lat gt lat_cent(i) and lat le lat_cent(i+1))
        enddo ! do i=1,nlat-1 
C put the theta and phi loops on the outside
C theta photon direction polar angle from geocenter loop
        if (ntheta.gt.12) then 
	  ntheta_loopmax=10 !cawh - to make code faster, Marcs matrix all zeros for theta>100.
	else 
	  ntheta_loopmax=ntheta   
        endif
        do itheta=1,ntheta_loopmax 
          theta_u=theta_cent(itheta)
C dont need theta weights and indicies since we will use each bin
C individually 

C phi photon direction around geocenter loop
          do iphi=1,nphi
            do ips=0,1 !here we are looping over + and - phi
              phi_u=phi_cent(iphi)
              if(ips .eq. 1) phi_u=-1.0*phi_u

C need to loop in both negative and positive phi to get all 
C the direct response matrices. atscat database access is the
C same for both negative and positive phi.

C load phi weights and indicies here

C also select the direct matrix at  this point
C calculate direction cosines dirx,diry,dirz, to  
C desired location in spacecraft coordaintes
              if (debug.gt.0) 
     .         print*,'theta_u, phi_u: ',theta_u, phi_u
              call geo_to_space (theta_u,phi_u,gx,gy,gz,
     .         dirx,diry,dirz,az,el)
              if (debug.gt.0)   
     .        print*,'dirx,diry,dirz: ',dirx,diry,dirz
c              if ((((abs(dirx-last_dirx).gt. 1.e-4).and.
c     .         (abs(diry-last_diry).gt. 1.e-4)).and.
c     .         (abs(dirz-last_dirz).gt. 1.e-4)).or.
c     .         ((itheta.eq.0).and.(iphi.eq.0))) then
              if (debug.gt.0) 
     .        print*,'Calling sky_interp for: ', az,el
C only interpolate a new matrix if the direction differes from the last one.     
              call sky_interp_lite (az, el, detector, 
     .		 drmdbpath, list, lptr, lend, x,y,z, 
     .           azimuth,zenith,milliaz,millizen, leaf_ver, n, maxen, 
     .           maxchan,debug,energ_lo, energ_hi,outmatrix,  
     .           edif_edge_lo,edif_edge_hi, ienerg, ichan,drmfiles)
              if (ienerg.ne.n_ein) then
	        print*,'Error! Atmospheric scattering database ',
     . 'and DRM database have incompatible dimensions. ',
     . 'Atscat db #photon energies = ',n_ein, 
     . ' DRM db  #photon energies = ',ienerg,
     . 'These values should be equal. Check atscat_file.'
                stop
	      endif  

c	      if (((itheta.eq.1).and.(iphi.eq.1))
c     .         .and.(ips.eq.0)) then
c                print*,drmfiles(1)
c                print*,'Back from sky interp. ienerg, ichan =',
c     .           ienerg,ichan
c                 do i=1,ienerg
c		  do j=1,ichan
c		    write(19,*) i,j,outmatrix(i,j),edif_edge_lo(i,j),
c     .               edif_edge_hi(i,j)	
c                  enddo
c		enddo  	    
c              endif 
              call echan_integrator (ienerg, maxen, maxchan, outmatrix,
     .	         edif_edge_lo, edif_edge_hi, ichan, ebin_edge_out,
     .           nobins_out, debug, direct_diff_matrix)

c	      if (((itheta.eq.1).and.(iphi.eq.1))
c     .         .and.(ips.eq.0)) then
c                do i=1,ienerg
c		  do j=1,nobins_out
c		    write(18,*) i,j,direct_diff_matrix(i,j)
c		  enddo
c		enddo
		
c	       endif	    	      
c              call sky_interp (az, el, detector, 
c     .		 drmdbpath, list, lptr, lend, x,y,z, 
c     .           azimuth,zenith,milliaz,millizen, n, maxen,maxchan, 
c     .           debug,energ_lo, energ_hi,outmatrix, edif_edge_lo, 
c     .           edif_edge_hi, ienerg, ichan,drmfiles)
c                 last_dirx = dirx
c		last_diry = diry
c		last_dirz = dirz
c                if (debug.gt.0)	  
C Now matrix mutiply the atscat db matrix for itheta, iphi by the direct matrix for 
C that direction. Interpolate only over latitude. Apply the abs(cos(lat)) correction
C factor. Interpolate only over latitude.   
                do i=1,ienerg
		  do j=1,nobins_out
		    do k=1,ienerg
		      atscat_diff_matrix(i,j) = atscat_diff_matrix(i,j)+
     .                (at_scat_data(i,k,il_low,itheta,iphi)*l_fract+
     .                  at_scat_data(i,k,il_high,itheta,iphi)*
     .                   (1.0-l_fract))*direct_diff_matrix(k,j)*
     .                    coslat_corr
c                     if (((itheta.eq.1).and.(iphi.eq.1)).and.
c     .                (ips.eq.0)) then
c                      write(17,*) i,j,k,
c     .                 atscat_diff_matrix(i,j),direct_diff_matrix(k,j)		
c                      endif       		            	
		    enddo !do k=1,ienerg - output side of scattered matrix, photon side of direct
		  enddo ! do j=1,nobins_out - output side of direct matrix
		enddo ! do i=1,ienergy - input side of scattered matrix. Scattered matrix is square.   			
            enddo! for ips=0,1 do begin
          enddo ! for iphi=0,nphi-1 do begin
        enddo !for itheta=0,ntheta-1 do begin
      endif! if( lat le lat_edge(nlat)) then begin
C interpolate input side of convolved scattered matrix to get user desired input
C edges      
      call atscat_highres_ephoton_interpolator (ebin_edge_in, nobins_in,
     .  e_in, ienerg, maxen, maxchan, atscat_diff_matrix, nobins_out,
     .  new_atscat_matrix)
c      close (12)
 500  continue !branch here if we've fallen off the edge of the scattering
               !matrix latitude values
      if (debug.eq.5) then
        do i=1,ienerg
          do j=1,nobins_out
            write(14,*) i,j,atscat_diff_matrix(i,j)
          enddo
        enddo
      endif		
      if (debug.ge.0) print*,'Returning from atscat_diff_calc_norm'
      return
      end		 
 
