       program gbmrsp
       implicit none
C *** gbmrsp v1.0 - Written by Colleen A. Wilson-Hodge NASA/MSFC
C 2007-Jul-31
C
C *** gbmrs v1.1 2007-Dec-12 Changes made to make_DRM_fits.for and
C  parse_drm_filename.f to accomodate changes to the input DRM database
C
C *** gbrsp v1.2 2008-March-11  
C Specific changes 
C 1. Fixed bug - for some sky positions, parse_drm_filename was not parsing
C a name that matched the database. azimuth and zenith in millidegrees were
C added to the triangulation info database file so the proper parsing could be 
C done.
C 2. Fixed bug - TRIPATH was not being used to point to the triangulation 
C  info database. Instead the database was read from the current directory.
C 3. Added matrix_type, atscat_dir, and atscat_file to input namelist
C (note, only matrix type 0 = direct works at this time)
C 4. Added drmdbpath and atscat_path to call to Make_drm_Fits
C 5. Added drmfiles as a variable returned from sky_interp.
C 6. Added atscat_file, drmfiles, and matrix_type to call to make_response
C
C *** gbmrsp v1.3 2008-March-14 implemented atmospheric scattering
C Specific changes 
C 1. matrix_type =0 (direct), 1(scattered), and 2(summed = direct+Scattered) now all
C supported.
C 2. Added call to read_atscat to read in atscat database if desired
C 3. Added call to atscat_diff_calc_norm - this code does the atmospheric scattering
C calculation by convolving results from the atmospheric scattering database for several
C scattering directions with the direct detector response matrix for each direction to
C create an atmospheric scattering matrix.
C 4. Added variables read_one_drm = 0 (no) or =1 (yes), one_drm_path, and one_drm_file
C  to namelist. This allows the option of inputing a specific drm instead of
C interpolating from the database.
C
C *** gbmrsp v1.4 2008-April-10 Added call to subroutine
C highres_ephoton_interpolator. This subroutine interpolates over input energies
C using the compressed matrix. Then the subroutine energy_integrator is called
C using the new input energies for both the input matrix and the desired matrix.
C This was a bug fix to smooth the response near the photopeaks in the case when
C the desired input energies were higher resolution that those in the database.
C Also, the call to make_response - replaced ethresh with ebin_edge_in and
C changed hardcoded 129 elements inside make response.
C
C *** gbmrsp v1.5 2008-April-15
C 1. Fixed bug in atscat_diff_calc_norm.f - e_edge(n_ener+1) is correct, not
C  e_edge(n_ener). g77 allowed the latter, even though I was writing to
C  element n_ener+1. gfotran did not allow writing past the end of an array.
C  Now the code compiles and produces identical results (within round-off
C  errors) using gfortran and g77.
C 2. Added trap to not allow input matrix energy edges that are outside of 
C the simulated range.

C *** gbmrsp v1.5t 2008-July-28  - test code, never released. (gbmrsp_test8.f)
C 1. Increased maximum number of matrices per file to 300 (maxpos=300)
C 2. Added trap to set scattering matrix to all zeros if lat>lat_cent(nlat) in
C     atscat_diff_calc_norm.f
C 3. Added normalizations for gx,gy,gz, and sl vectors in geocords.f
C 4. Reversed order of phi_cent.
C 5. Changed units of input direct matrix in atscat_diff_calc_norm_test2.f from
C  cm^2/keV to cm^2. Geoff says atscat_diff_calc_norm should output a scattered
C matrix in the same units as the input drm. I want cm^2 for later in gbmrsp.
C 6. Removed multiplication of atscat_diff_matrix by energy bin width in
C gbmrsp_test5.f
C 7. calls atscat_diff_norm_test3.f which calls sky_interp_lite.f instead of
C sky_interp.f. For each scattering point, this will grab the nearest drm from
C Marc's database instead of reading in 3 drms and interpolating. This should
C speed the code up considerably.
C 8. Added if statements to turn off standard messages if debug < 0
C
C *** gbmrsp v1.6 2008-September-3
C Major changes:
C Atmospheric scattering was not working properly. Extensive testing revealed that
C (1) The definition of phi in Geoff's database was 180 degrees of what this code
C expected and (2) Geoff's database predicted maximum scattering near latitude = 90
C deg, which disagrees with BATSE extrapolations (3) The scattering calculation
C predicted way too many counts at all angles. 
C Problem (1) was addressed by changing the definition of phi_edge in gbmrsp.f
C Problem (2) was addressed by multiplying the input atmospheric scattering matrix by
C cos(latitude) where latitude is defined as the angle between the source and the
C geocenter. Latitude is 0 for a source that is directly overhead and 90 near the
C horizon.
C Problem (3) was addressed by the following changes:
C (a) In atscat_diff_matrix.f, the input direct matrix is no longer divided by energy
C     i.e. the units were changed from cm^2/keV to cm^2 because cm^2 is the desired
C     units later in the code.
C (b) In gbmrsp.f the scattered matrix is no longer multiplied by energy. Previously 
C this step was to convert the units of the scattered matrix back to cm^2.
C Minor changes:
C (1) Added debug=-1 option. This turns off all messages to the screen except error
C messages.
C (2) Increased the maximum number of responses per file from 100 to 300.
C (3) In atscat_diff_matrix.f - this now uses the closest drm from Marc's database for
C the scattering calculation. No spatial interpolation is done.
C (4) In geocords.f - the vectors gx,gy(part of conversion from scattering database
C     coordinates to spacecraft coordinates),gz (geocenter direction),and 
C     sl(source direction) are now normalized
C
C gbmrsp v1.7 2008-November-24
C Major Change:
C (1) New approach for atmospheric scattering, where database is
C     pre-interpolated in energy to match photon edges for the GBM leaf drms
C     DRMs. Then the appropriate atmospheric scattering matrix is matrix
C     multiplied by the direct matrix for that direction, after each direct
C     matrix has been uncompressed. The scattered matrix is then interpolated
C     over the user photon energies.
C     Routines changed: atscat_diff_calc_norm.f, gbmrsp.f
C     Routines added: read_atscat_alldbs.f, get_atscatdb_dimensions.f,
C		      atscat_highres_ephoton_interpolator.f
C     Routines deleted: read_atscat.f, samp_calc.f
C Additions: 
C (1) Audit trail information for LUTs has been added to the namelist and 
C      EBOUNDS extension of the output rsp files.
C (2) The version number for the DRM leaf files has been added to the namelist
C     and is now passed to sky_interp and atscat_diff_calc_norm to be used by
C     parse_drm_filename. 
C (3) Trap added to highres_ephoton_interpolator.f to handle cases where 
C     desired energy and original energy are basically equal but not quite.
C (4) energy_integrator.f was replaced with echan_integrator.f.
C      Energy_interpolator.f interpolated over both the ephoton and echannel
C      energies. Since this step is done in highres_ephoton_integrator, it was
C      redundant and was removed.
C (5) A bug was fixed in calc_sphere_dist.f. It was calculating incorrect
C      distances when the elevation angle was large.
C
C gbmrsp v1.8pr (prerelease)
C Major changes:
C A bug of omission was found - the responses needed to be integrated over
C photon bins. To do this, the code now computes interpolated values for the
C response at each requested photon bin and at the logarithmic center of each 
C bin. Each bin is then integrated over 3 points (lower edge, logarithmic
C center, and upper edge) using the trapezoidal rule. CCR#192
C Minor changes:
C (1) echan_integrator.f code was replaced with echan_integrator.REV.f - the code
C was cleaned up by Michael Briggs to make it more readable. 
C (2) sec_to_date.f was changed so that it no longer outputs values of 60
C seconds in month-day-hours-min-secs keywords in the headers.
C   CCR #177
C (3) CCR #193 - calling_code (string) was added to the namelist file. This is
C  expected to contain the calling code name and version number and is appended
C  to the CREATOR keyword in the fits file. In previous versions the CREATOR
C  keyword was hardcoded to GBMRespGen (doesn't exist). This was corrected to be
C  GBMRSP. 
C
C gbmrsp v1.8rc (release candidate) September 16, 2009 CAWH
C (1) Another bug was found in sec_to_date.f that caused the seconds to be negative. This crashed
C  later fits utility code which caused fewer than expected matrices to be written to the file.
C (2) The limit on the # of photon energy edges was set back to 200 (it was 300 in 1.8pr), because
C   no improvement was apparent in fits to grb090902b when more than 199 edges were used.
C (3) Now the environment variable GBMRSP_NML must be set by the user to point 
C   to the namelist file.
C
C gbmrsp v1.8 (release) September 28, 2009 - contains changes listed under
C  gbmrsp v1.8pr and gbmrsp v1.8rc. (max photon energies is 200) .
C  This version should close CCR #177, #192, #193.
C
C gbmrsp v1.81 (release) October 13, 2009 - tiny bug fix that only affects 
C  read_one_drm=1 special mode (no effect for most users). The order of maxen
C and maxchan was reversed in the call to readpixeedges from read_single_drm.
C This bug was present in versions prior  v1.8, but had no effect because
C maxen=maxchan. In v1.8, maxen was not equal to maxchan, so read_one_drm=1 mode
C produced matrices with only NaN values and zeros.
C
C gbmrsp v1.9 (release) December 29, 2009 - added DET_ANG and GEO_ANG keywords
C  to SPECRESP MATRIX extension headers. DET_ANG = angle between source and 
C  detector normal. GEO_ANG = angle between geocenter and detector normal.
C  For the BGOs that don't exactly have detector normals, I used the +x and -x
C  axes for BGO 0 and 1 respectively. Also, I corrected the comments in the
C  SPECRESP MATRIX headers for SRC_AZ, SRC_EL, GEO_AZ, and GEO_EL - they now
C correctly say that these quantities are in SPACECRAFT coordinates (instead of
C detector coordinates as they previously said.
C
C gbmrsp v2.0t March 3, 2011 - changed the allowed number of input photon bins
C  from 200 to 3000. Also changed maxen from 402 to 6002. Also changed allowable
C  length of output rsp file name from 120 to 240 characters. 
C
C gbmrsp v2.0 (release) Nov 3, 2011. Addresses CCR #379 - changed the allowed 
C  number of input photon bins from 200 to 3000. Also changed maxen from 402 to 
C  6002. Changed allowable length of output rsp file name from 120 to 240 characters
C  Also addresses CCR #203 - check that photon and channel energies are strictly increasing.
C
C General:
C This code generates direct response matrices for each input 
C source position in spacecraft coordinates. 
C This code will do the following once per call
C (1) Read or perform the triangulation of the sky locations in the response
C database.
C (2) Write the primary header and the EBOUNDS extension for the output fits
C file
C(3) For each source position (s/c az, el) this program will:
C     (a) Interpolate the response database, using the 3 points in the triangle
C         containing the source position in S/C coordinates.
C     (b) Integrate over energy to generate the desired direct response matrix
C     (c) Write the matrix and photon edges to a new SPECTRESP extension in the
C         FITS file.
C This code will read in the following parameters from a namelist file
      integer debug ! if debug >=1 increases the verbosity of the main code
                    ! if debug >=2 increases verbosity of main code and 
		    ! subroutines.
		    ! if debug <0 - verbosity is suppressed so that only error
		    ! messages are output.
C Inputs for the triangulation of the response database
C The following inputs (related to the triangulation) are included in the input
C  namelist file:
C detector = detector number 0-13. detectors 0-11 are NaIs and 12-13 are BGOs
C triflag = flag denoting whether to read =0, perform = 1 the triangulation.
C tripath,trifile = the path and file name for the triangulation fits file
C infile = an ascii list of positions at which response files are in the
C          database this is used only if triflag =1 
      integer detector, triflag
      character*80 tripath, trifile
      character*100 infile      
C The following parameters, related to the angular sky interpolation will be
C read in from the namelist file.
C src_ra, src_dec: The source right ascension and declination.
C npos: The number of different s/c source positions
C src_az(npos), src_el(npos): The source position in s/c coordinates 
C geo_az(npos), geo_el(npos): The geocenter position corresponding to 
C  each source position
C tstart(npos),tstop(npos): the time range corresponding to each position
C trigger_sec: The trigger time for the GRB
C drmdbpath: The path to the drm database
C leaf_ver: The version number for the drm leaf files in the database.
      integer npos, maxpos
      parameter (maxpos=300)
      real src_ra, src_dec !These are just passed to the fits file
      real src_az(maxpos), src_el(maxpos), geo_az(maxpos),geo_el(maxpos)
      character*80 drmdbpath
      character*3 leaf_ver
C The following parameters are used by make_DRM_fits
C tstart(npos),tstop(npos): the time range corresponding to each position
C trigger_sec: The trigger time for the GRB
C history_name: the name of the output response file
      character*240 history_name 
C energy_calib_scheme: The name of the energy calibration scheme
      character*120 energy_calib_scheme
C File name for LUT used in contrsuction of the EBOUNDS array
      character*120 lut_filename
C GBM FSW icrc checksum value (hex) for the LUT
      character*20 lut_checksum
C Gain correction factor applied to the energy_edges      
      real         gain_cor
C object_class: i.e. GRB, BHC, pulsar, XRB, AGN, etc.
      character*20 object_class
      real*8 trigger_sec,tstart(maxpos),tstop(maxpos)      
C The following parameters, related to the energy interpolation will be read in
C from the namelist file:
C nobins_in, nobins_out: The number of photon and channel edges in the desired
C  output matrix.
C ebin_edge_in(nobins_in): The photon edges for the output matrix
C ebin_edge_out(nobins_out): The energy channel edges for the output matrix.
C  Note, this may be replaced with the LUT and the conversion done inside this
C  code.
      integer nobins_in, nobins_out
      real ebin_edge_in(3000), ebin_edge_out(129)
C The following parameters related to inclusion of atmospheric 
C scattering will be read in from the namelist file.
C matrix_type: 0 output direct matrices only,
C              1 output scattering matrices only
C              2 output summed (direct+Scattered) matrices only
C atscat_path: The path to the atmopheric scattering database
C atscat_file: The filename containing the atmospheric scattering db
      integer matrix_type
      character*80 atscat_path
      character*40 atscat_file   
      integer use_coslat_corr ! right now I believe it is correct to multiply the
                              ! scattered matrix by the cosine of the angle between the 
			      ! source direction (in spacecraft coordinates) and the
			      ! geocenter direction(in spacecraft coordinates). This 
			      ! flag allows the user to turn this correction on or off.
C read_one_drm = 0 (no) = 1 (yes) - this allows the user to select the option to input
C one specific drm rather than interpolating from the database. 
C one_drm_path, one_drm_file - the path name and file name for the specific drm file.
      integer read_one_drm
      character*80 one_drm_path
      character*40 one_drm_file 
C calling_code -  this is a string containing the name of the calling code, e.g. 
C  "SA_GBM_RSP_Gen.pl V1.2". This will be passed to make_DRM_FITS.for to be appended to the 
C CREATOR keyword in the fits file.
      character*30 calling_code                 
C Define input namelist
      namelist /gbmrsp_inputs/ debug, detector, triflag, tripath, 
     . trifile, infile, src_ra, src_dec, drmdbpath,nobins_in,nobins_out, 
     . ebin_edge_in, ebin_edge_out, npos, src_az, src_el, geo_az,geo_el,
     . tstart,tstop,trigger_sec,history_name,energy_calib_scheme,
     . object_class, matrix_type, atscat_path, atscat_file,
     . use_coslat_corr, read_one_drm, one_drm_path, one_drm_file, 
     . lut_filename, lut_checksum, gain_cor, leaf_ver,calling_code
C Local variables needed for the triangulation and sky interpolation codes                  
C get_triangulation
      integer n, nmax, nmx6
      parameter (NMAX=1500, NMX6=6*NMAX)
      integer  LIST(NMX6), LPTR(NMX6), LEND(NMAX)
      real X(NMAX),Y(NMAX),Z(NMAX),Azimuth(NMAX),Zenith(NMAX)
      integer milliaz(nmax), millizen(nmax)
C Local variables needed for the sky interpolation code sky_interp
      integer maxen, maxchan,ienerg, ichan
      parameter (maxen=6002, maxchan=128)
      real energ_lo(maxen),energ_hi(maxen),
     . outmatrix(maxen,maxchan),edif_edge_lo(maxen,maxchan),
     . edif_edge_hi(maxen,maxchan)
      character*40 drmfiles(3)
C Local variables needed for interpolation and integration of the photon side of the matrix
      integer n_tmp_phot_bin
      real tmp_phot_bin(maxen), tmp_matrix(maxen,maxchan)     
C Local variables needed to read the atmospheric scattering database
C with subroutine read_atscat and to perform the atmospheric scattering
C calculations with atscat_diff_calc_norm. See atscat_diff_calc_norm for
C detailed comments.   
      integer n_ein, n_ener, nlat, ntheta, nphi, n_ener_abin,n_ener_low
      real*4 at_scat_data(70,70,18,18,18), e_in(70), lat_edge(19)
      real*4 lat_cent(18), theta_cent(18), phi_cent(18)
      real*4 atscat_diff_matrix(maxen,maxchan)
      real*4 theta_source, phi_source, theta_geo, phi_geo
C Local variables needed for the energy interpolation code 
C energy_integrator
      real ein(maxen), diff_matrix(maxen,maxchan), 
     .  binned_matrix(maxen,maxchan),alfa,
     . new_epx_lo(maxen,maxchan),new_epx_hi(maxen,maxchan)
C local variables for make_DRM_FITS
      real*8 time_range(2)
      real source_location(2)  
      character*6 data_type
      integer status   
      character*80 message
      integer*2 nchannels
C local variables for Make_Response
      real source_az_el(2), earth_location(2)
c     .     ethresh(129) 
C local variables for the main code
      integer lnml,i,j,k
      real rlon, rlat
      character*120 gbmrsp_nmlfile
C******************************************************************************
C begin executable code
C******************************************************************************       
      lnml=1
      call getenv('GBMRSP_NML',gbmrsp_nmlfile) 
      open (lnml,file=gbmrsp_nmlfile,status='OLD',err=199)
      read (lnml,nml=gbmrsp_inputs,err=299) 
      close (lnml)
      if (debug.gt.0)then
        print*,'Namelist inputs: '
        print*,'detector, triflag: ',detector, triflag
        print*,'tripath,trifile: ',tripath,trifile
        print*,'drmdbpath: ',drmdbpath
	print*,'leaf_ver: ',leaf_ver
        print*,'src_ra,src_dec: ',src_ra, src_dec
        print*,'nobins_in, nobins_out: ',nobins_in,nobins_out
        print*,'ebin_edge_in: ',(ebin_edge_in(i),i=1,nobins_in+1)
        print*,'ebin_edge_out: ',(ebin_edge_out(i),i=1,nobins_out+1)
        print*,'npos = ',npos
        print*,'src_az: ',src_az
        print*,'src_el: ',src_el
        print*,'geo_az: ',geo_az
        print*,'geo_el: ',geo_el
        print*,'tstart: ',tstart
        print*,'tstop: ',tstop
        print*,'trigger_sec: ',trigger_sec
        print*,'history_name: ',history_name
        print*,'energy_calib_scheme: ',energy_calib_scheme
	print*,'lut_filename: ',lut_filename
	print*,'lut_checksum: ',lut_checksum
	print*,'gain_cor: ',gain_cor
        print*,'object_class: ',object_class
	print*,'matrix_type: ',matrix_type
	print*,'atscat_path: ',atscat_path
	print*,'atscat_file: ',atscat_file
	print*,'read_one_drm: ',read_one_drm
	print*,'one_drm_path: ',one_drm_path
	print*,'one_drm_file: ',one_drm_file
      endif	
      if ((detector.lt.12).and.((ebin_edge_in(1).lt.3.0).or.
     . (ebin_edge_in(nobins_in+1).gt.73236.164))) then
        print*,'***Error***'
        print*,
     .'Input energy edges outside of simulated range for detector ',
     .  detector
       print*,'Extrapolation beyond simulated range is not implemented.'
        stop
      endif	  
      if ((detector.ge.12).and.((ebin_edge_in(1).lt.50.12).or.
     . (ebin_edge_in(nobins_in+1).gt.267539.1))) then 
        print*,'***Error***'
        print*,
     .'Input energy edges outside of simulated range for detector ',
     .  detector
       print*,'Extrapolation beyond simulated range is not implemented.'
        stop
      endif 
C Check that photon bin edges and channel bin edges are strictly increasing in energy.
      do i=1,nobins_in-1
        if (ebin_edge_in(i).ge.ebin_edge_in(i+1)) then
	  print*,'***Error***'
	  print*,'Input photon edges must be strictly increasing.',i,ebin_edge_in(i),i+1,ebin_edge_in(i+1)
	  stop
	endif
      enddo
      do i=1,nobins_out-1
        if (ebin_edge_out(i).ge.ebin_edge_out(i+1)) then
	  print*,'***Error***'
	  print*,'Energy channel edges must be strictly increasing.',i,ebin_edge_out(i),i+1,ebin_edge_out(i+1)
	  stop
	endif
      enddo
	  	        
C If the user has specified read_one_drm = 1, go ahead and read in the specific drm. Note, this is not the
C cleverest code at this point. It reads in the specified drm, but expects that the user has input the 
C correct values into the namelist for src_az, src_el, and detector. It does not read those values from the
C input drm file.
      if (read_one_drm.eq.1) then
        call read_single_drm (one_drm_path,one_drm_file,maxen, maxchan,
     .   energ_lo, energ_hi, outmatrix, ichan, ienerg, edif_edge_lo,
     .   edif_edge_hi)
        print*,'Finished reading specific DRM. ',
     .   'one_drm_path = ',one_drm_path, 
     .   'one_drm_file = ',one_drm_file
        if (matrix_type.eq.0) drmdbpath = one_drm_path
	drmfiles(1) = one_drm_file
	drmfiles(2) = 'none'
	drmfiles(3) = 'none'
	npos = 1
      endif	  	    
C Write the fits header and EBOUNDS extension 
C some silly parameters for testing
c      trigger_sec=2.07072930D+08
      source_location(1)=src_ra
      source_location(2)=src_dec
      time_range(1)=tstart(1) !2.07072930D+08
      time_range(2)=tstop(1) !2.07072940D+08
c      history_name='testdrmout.fits'
c      energy_calib_scheme='Imadeitup'
c      object_class='Idunno'
      status=0
      if (nobins_out.eq.8) then
        data_type='CTIME '
c	do i=10,129
c	  ebin_edge_out(i)=0.
c	enddo  
      else if (nobins_out.eq.128) then
        data_type='CSPEC '
      else
        data_type='UNDEF '
      endif
c      nchannels = nobins_out

c      print*,'nchannels=',nchannels,nobins_out
      if (debug.ge.0) print*,'Creating and writing FITS headers for:',
     . history_name 	
c          print*,'drmdbpath = ',drmdbpath
      call make_DRM_FITS(trigger_sec,data_type,source_location,
     . nobins_out,time_range,ebin_edge_out,
     . detector,history_name,energy_calib_scheme,
     . lut_filename, lut_checksum,gain_cor,
     . object_class, drmdbpath, atscat_path,calling_code,
     . debug,status, message)
      if (debug.gt.0) 
     . print*,trigger_sec,data_type,source_location,
     . nobins_out,time_range,ebin_edge_out,
     . detector,history_name,energy_calib_scheme,
     . object_class,drmdbpath, atscat_path, status
      if (status.ne.0) then
        print*,'status = ',status,message
	stop
      endif
C read/perform the triangulation         
      call get_triangulation (infile, tripath, trifile, triflag,
     . debug,list,lptr,lend,x,y,z,azimuth,zenith,milliaz,
     . millizen,n)
      if (debug.ge.0) 
     . print*,'Finished reading triangulation info. n = ',n
C If we are going to do scattering calculations, read in the
C atmospheric scattering database.
      if(matrix_type.ge.1) then
        call get_atscatdb_dimensions(atscat_path,
     .   atscat_file, n_ein, nlat, ntheta, nphi) 
        n_ener=n_ein	
        call read_atscat_alldbs(atscat_path,atscat_file,
     .   detector, at_scat_data, e_in,
     .   lat_edge,lat_cent,theta_cent, phi_cent, n_ein, n_ener, nlat, 
     .   ntheta, nphi)
c       print*,'n_ein,n_ener, nlat, ntheta, nphi: ',
c     .    n_ein,n_ener, nlat, ntheta, nphi
c       print*,'at_scat_data(5,*,9,8,1): ',
c     .   (at_scat_data(5,i,9,8,1),i=1,n_ener)
c       stop	
C reverse order of phi edges (suggested by Geoff 7/29/08)
        do i=1,nphi
	  phi_cent(i)=180.-phi_cent(i)
	enddo       
        if (debug.ge.0) print*,'Read atmospheric scattering database'
      endif
C Loop over each source position in S/C coordinates
c      npos = 2
c      src_az(1)=src_ra
c      src_el(1)=src_dec
c      src_az(2)=src_ra
c      src_el(2)=src_dec-2.0
      do i=1,npos
        rlon = src_az(i)
	rlat = src_el(i)
C note, sky_interp has a few simple changes to the call: remove use_test_data,
C  testeffarea, echan_min, echan_max. Bigger change: add code to read edif_edge_lo,
C  edif_edge_hi  
C sky_interp interpolates the direct detector response from the database
C for the selected sky position.
c        If ((matrix_type.eq.0).or.(matrix_type.eq.2)) then
        if (read_one_drm.ne.1) then 
c          if (debug.ge.0) 
           print*,'Calling sky_interp for source position(az,el):',
     .     rlon,rlat
c          print*,'drmdbpath = ',drmdbpath
          call sky_interp (rlon,rlat, detector,drmdbpath,
     .      list,lptr,lend,x,y,z,azimuth,zenith,milliaz,millizen,
     .      leaf_ver, n,maxen,maxchan,debug, 
     .      energ_lo,energ_hi, outmatrix, edif_edge_lo, edif_edge_hi, 
     .      ienerg,ichan,drmfiles)	  
c          print*,'drmdbpath = ',drmdbpath
          if (debug.gt.0) then   
            do j=1,ienerg
	      print*,'j,energ_lo(j),energ_hi(j) = ',
     .        j,energ_lo(j),energ_hi(j) 
              do k=1,ichan
	        print*,
     .        'j,k,outmatrix(j,k),edif_edge_lo(j,k),edif_edge_hi(j,k):',        
     .         j,k,outmatrix(j,k),edif_edge_lo(j,k),edif_edge_hi(j,k)
              enddo
	    enddo
	  endif  
	endif
c initialize the integrated binned matrix
        do j=1,nobins_in
          do k=1,nobins_out
	    binned_matrix(j,k)=0.
          enddo
        enddo	 
C Interpolate over photon energies first
C make local array of 2*desired photon energies, including points at the logarithmic center of each 
C photon bin
        n_tmp_phot_bin = 2*nobins_in+ MOD(nobins_in, 2)
c	print*,nobins_in,n_tmp_phot_bin
	do j=1,n_tmp_phot_bin-1,2
	  tmp_phot_bin(j) = ebin_edge_in(j/2+1)
	  tmp_phot_bin(j+1)=10**((alog10(ebin_edge_in(j/2+1))+
     .      alog10(ebin_edge_in(j/2+2)))/2.)
c          print*,j,j/2+1, tmp_phot_bin(j)
c	  print*,j+1,tmp_phot_bin(j+1)     
        enddo
	if (j.eq.n_tmp_phot_bin) then
	  tmp_phot_bin(j)=ebin_edge_in(nobins_in+1)
c	  print*,j,tmp_phot_bin(j)
	endif  

C get the atmospheric scattering matrix for the selected source and
C geocenter positions. 	
	if ((matrix_type.eq.1).or.(matrix_type.eq.2))then
	  if (debug.ge.0) print*,'Calling atscat_diff_calc_norm'
c          print*,'drmdbpath = ',drmdbpath
	  theta_geo=90.-geo_el(i)
	  phi_geo = geo_az(i)
	  theta_source = 90.-rlat
	  phi_source = rlon
c Initialize scattered matrix	  
	  do j=1,n_tmp_phot_bin-1
	    do k=1,nobins_out
	      atscat_diff_matrix(j,k) = 0.
	    enddo
	  enddo    
	  
          call atscat_diff_calc_norm ( n_ein,n_ener,nlat,ntheta,nphi,
     .     at_scat_data, e_in,lat_edge, lat_cent,theta_cent,phi_cent, 
     .     use_coslat_corr,theta_geo,phi_geo,
     .     theta_source,phi_source,detector,drmdbpath, 
     .     list,lptr,lend,x,y,z, azimuth, zenith, milliaz,millizen,  
     .     leaf_ver, n, maxen,maxchan, debug, energ_lo,energ_hi, 
     .     edif_edge_lo,edif_edge_hi, ienerg, ichan,  ebin_edge_out, 
     .     nobins_out, tmp_phot_bin, n_tmp_phot_bin, atscat_diff_matrix)	
c          print*,'drmdbpath = ',drmdbpath
        endif
C interpolate input (photon) side of direct matrix	       	
	if ((matrix_type.eq.0).or.(matrix_type.eq.2)) then
          do j=1,ienerg
	    ein(j)=energ_lo(j)
	  enddo
	  ein(ienerg+1)=energ_hi(ienerg)	  	

	  call highres_ephoton_interpolator(tmp_phot_bin,n_tmp_phot_bin,ein,
     .     ienerg,maxen,maxchan,outmatrix,edif_edge_lo,edif_edge_hi,
     .     ichan,new_epx_lo,new_epx_hi,diff_matrix)		     	
C Integrate/Interpolate over output bins.
          if (debug.ge.0) print*,'Calling echan integrator'
          call echan_integrator (n_tmp_phot_bin,maxen,maxchan,diff_matrix,
     .     new_epx_lo, new_epx_hi,ichan,ebin_edge_out,
     .     nobins_out, debug, tmp_matrix)
c          do j=1,nobins_in
c	    do k=1,nobins_out
c              write(9,*) j,k, binned_matrix(j,k)
c	    enddo
c	  enddo     
          if (debug.ge.0) print*, 'Returned from echan integrator'   
        endif
C Combine the direct and scattered matrices if user desires. 
C       if (matrix_type.eq.0) !direct only
C  we are all set to write to the fits file. Do nothing

	if (matrix_type.eq.1) then !scattered only
	  do j=1,n_tmp_phot_bin-1
	    do k=1,nobins_out
	      tmp_matrix(j,k)=atscat_diff_matrix(j,k)
            enddo
	  enddo
        endif
	if (matrix_type.eq.2) then !direct+scattered
	  do j=1,n_tmp_phot_bin-1
	    do k=1,nobins_out
	      tmp_matrix(j,k)=tmp_matrix(j,k)+
     .         atscat_diff_matrix(j,k)
            enddo
	  enddo
        endif
C Integrate the final matrix over photon energies
        call ephoton_integrator(tmp_phot_bin,n_tmp_phot_bin,tmp_matrix,
     .     nobins_out,nobins_in,ebin_edge_in, maxen,maxchan, binned_matrix)    

C write SPECRESP extension to FITS file. 
        source_az_el(1) = rlon
	source_az_el(2) = rlat
	earth_location(1) = geo_az(i)
	earth_location(2) = geo_el(i)
	time_range(1)=tstart(i)
	time_range(2)=tstop(i)
	if(debug.gt.1) then
          do j=1,nobins_in
	    do k=1,nobins_out
	      write(i+10,*) j,k,binned_matrix(j,k)
	    enddo
  	  enddo
	endif
	status =0 	
	if (debug.ge.0) print*,'Writing SPECRESP extension to ',
     .   history_name
        call make_response(trigger_sec, source_location, 
     .   source_az_el,earth_location, nobins_out,nobins_in,
     .   time_range,ebin_edge_in,detector,history_name,
     .   binned_matrix, maxen,maxchan,
     .   object_class, drmfiles, atscat_file, matrix_type,
     .   debug,status, message)
        if (status.ne.0) then
	  print*,'status = ',status, message
	  stop
	endif
	if (debug.gt.1) then
	  print*,'Matrix upon exit from make_response: i = ',i
          do j=1,nobins_in
	    do k=1,nobins_out
	      write(*,*) j,k,binned_matrix(j,k)
	    enddo
	  enddo
	endif  
      enddo
      goto 500
 199  write (*,*) 'Error opening gbmrsp.nml'  
      stop
 299  write (*,*) 'Error reading gbmrsp.nml'
      stop
 500  if (debug.ge.0) write (*,*) 'Successful completion.'
      end
       
       
       
