      subroutine highres_ephoton_interpolator (ebin_edge_in, nobins_in,
     . ein, nvbins, maxen, maxchan, matrix, edif_edge_lo, edif_edge_hi,
     . nhbins,new_epx_lo,new_epx_hi,diff_matrix)
C Written by Colleen Wilson-Hodge NASA/MSFC 2008-April-10.
C This code interpolates over the input photon edges, directly using the
C compressed matrices from the LANL database. In that database, the
C photopeaks are already aligned, so interpolating between them is
C simple.
      implicit none
C inputs
      integer nobins_in, !number of desired bins on input side of diff_matrix
     . nvbins, !number of input bins for matrix
     . maxen, !maximum input bins for matrix or diff_matrix
     . maxchan, !maximum number of output bins for matrix or diff_matrix
     . nhbins !number of output bins for matrix and diff_matrix
      real ebin_edge_in(nobins_in+1), !Desired input bin edges
     . ein(nvbins+1), !original input bin edges
     . matrix(maxen,maxchan), !original matrix
     . edif_edge_lo(maxen,maxchan), !Lower Energy edges for each matrix pixel
     . edif_edge_hi(maxen,maxchan) !Upper Energy edges for each matrix pixel      
C outputs
       real diff_matrix(maxen,maxchan), !interpolated matrix
     .  new_epx_lo(maxen,maxchan), !Interpolated pixel lower energy edges          
     .  new_epx_hi(maxen,maxchan) !Interpolated pixel upper energy edges
C local variables
       integer i,j,k, ivfind
       real mu
c       do i=1,nobins_in
c         do j=1,nhbins
c	   diff_matrix(i,j)=0.0
c	 enddo
c       enddo 	   
       ivfind=1
       do i=1,nobins_in  
         do j=ivfind, nvbins
           if((ebin_edge_in(i).ge.ein(j)).and.
     .        (ebin_edge_in(i).lt.ein(j+1))) then
             ivfind=j
	     mu=(log(ebin_edge_in(i))-log(ein(ivfind)))/
     .        (log(ein(ivfind+1))-log(ein(ivfind)))
             if ((mu.lt.0.0).and.(mu.gt.-1e-5)) mu=0.0
	     if ((mu.gt.1.0).and.(mu.lt.1.00001)) mu=1.0
	     if ((mu.lt.-1e-5).or.(mu.gt.1.00001)) then
	       print*,'Inside highres_ephoton_interpolator:',
     .          ' Unexpected value for mu: ',mu
               stop
	     endif  
c          write(16,*) i,ivfind,ebin_edge_in(i),
c     .     ein(ivfind),ein(ivfind+1),mu
             do k=1,nhbins
               new_epx_lo(i,k)=(edif_edge_lo(ivfind,k)/ein(ivfind)*
     .          (1-mu)+edif_edge_lo(ivfind+1,k)/ein(ivfind+1)*mu)*
     .          ebin_edge_in(i)
               new_epx_hi(i,k)=(edif_edge_hi(ivfind,k)/ein(ivfind)*
     .          (1-mu)+edif_edge_hi(ivfind+1,k)/ein(ivfind+1)*mu)*
     .          ebin_edge_in(i)
	       diff_matrix(i,k)=(matrix(ivfind,k)*(1-mu)+
     .	        matrix(ivfind+1,k)*mu)!/(new_epx_hi(i,k)-new_epx_lo(i,k))
C change - 5-November-2008. Removed divide by energy. That step is now done in
C echan_integrator.f     
c               write(15,*) i,k,ivfind, 
c     .          matrix(ivfind,k), matrix(ivfind+1,k),
c     .          diff_matrix(i,k),new_epx_lo(i,k),
c     .          new_epx_hi(i,k)
             enddo
	   else if ((j.eq.nvbins).and.((new_epx_lo(i,1).eq.0.0).and.
     .      (new_epx_hi(i,1).eq.0.0))) then 
             print*, 'Bin not found. ebin_edge_in(',i,') = ',
     .        ebin_edge_in(i)
	     print*, 'ein(',nvbins,') = ',ein(nvbins), 
     .        ' ein(',nvbins+1,') = ',ein(nvbins+1)
             stop
	   endif 
	 enddo  
       enddo	   
       return
       end	      
	     	     
