      SUBROUTINE INTRC0SP (B1,B2,B3,I1,I2,I3,ienerg,ichan,
     .  maxen, maxchan,matrix1,matrix2,matrix3,outmatrix,ier)
      implicit none
      INTEGER I1,I2,I3,ienerg,ichan, IER,i,j,
     .  maxen,maxchan
      REAL    B1,B2,B3,SUM, matrix1(maxen,maxchan),
     .     matrix2(maxen,maxchan),matrix3(maxen,maxchan),
     .     outmatrix(maxen,maxchan),b1n,b2n,b3n
C     
C This code has been modified from Renka's original INTRC0 by
C Colleen Wilson-Hodge NASA/MSFC 2007-Jul-31
C (1) The call to TRFIND has been moved outside the routine
C (2) It is assumed that the point is within a triangle.
C (3) The code that does the actual interpolation has been 
C     changed so that 3 matrices are input and one matrix
C     is output.
C An excerpt from the original documentation of INTRC0 is below.
C***********************************************************
C
C                                              From SSRFPACK
C                                            Robert J. Renka
C                                  Dept. of Computer Science
C                                       Univ. of North Texas
C                                           renka@cs.unt.edu
C                                                   07/24/96
C
C   Given a triangulation of a set of nodes on the unit
C sphere, along with data values at the nodes, this sub-
C routine computes the value at a point P of a continuous
C function which interpolates the data values.  The interp-
C olatory function is linear on each underlying triangle
C (planar triangle with the same vertices as a spherical
C triangle).  If P is not contained in a triangle, an ex-
C trapolated value is taken to be the interpolated value at
C the nearest point of the triangulation boundary.
C**********end of excerpt from Renka documentation**************
C On input
C B1,B2,B3 = Barycentric coordinates of the central projec-
C              tion of P onto the underlying planar trian-
C              gle, or (B1 and B2) projection of Q onto the
C              underlying line segment N1-N2 when P is
C              exterior.  Unnormalized coordinates are
C              computed by TRFIND when P is in a triangle.
C I1,I2,I3 = Vertex indexes returned by TRFIND
C ienerg, ichan = the number of photon energies and channels contained
C  within each matrix
C matrix1,matrix2,matrix3 - the response matrices at the triangle vertices.
C On output:
C OUTMATRIX- the interpolated response matrix
C
C       IER = Error indicator:
C             IER = 0 if interpolation was performed
C                     successfully.
C            IER = -2 if the nodes are collinear.
C             IER = -3 if P is not in a triangle and the
C                      angle between P and the nearest boun-
C                      dary point is at least 90 degrees.
C
C
C***********************************************************
C
C Local parameters:
C B1n,B2n,B3n = Normalized Barycentric coordinates of the central projec-
C              tion of P onto the underlying planar trian-
C              gle, or (B1 and B2) projection of Q onto the
C              underlying line segment N1-N2 when P is
C              exterior.  Unnormalized coordinates are
C              computed by TRFIND when P is in a triangle.
C SUM =      Quantity used to normalize the barycentric
C              coordinates
      IF (I1 .EQ. 0) GO TO 12
      IF (I3 .NE. 0) THEN
C
C P is contained in the triangle (I1,I2,I3).  Normalize the
C   barycentric coordinates.
C
        SUM = B1 + B2 + B3
        B1n = B1/SUM
        B2n = B2/SUM
        B3n = B3/SUM
        do i=1,ienerg
	  do j=1,ichan	
            outmatrix(i,j) = B1n*matrix1(i,j) + 
     .	     B2n*matrix2(i,j) + B3n*matrix3(i,j)
c            if (j.eq.11) write(*,*) i,j,b1n,'*',matrix1(i,j),'+',
c     .       B2n,'*',matrix2(i,j),'+',b3n,'*',matrix3(i,j),'=',
c     .       outmatrix(i,j)
	  enddo
	enddo    
        IER = 0
        RETURN
      ENDIF
C
C
C Collinear nodes.
C
   12 IER = -2
      RETURN
C
C The angular distance between P and the closest boundary
C   point to P is at least 90 degrees.
C
   13 IER = -3
      RETURN
      END
