      SUBROUTINE Make_DRM_FITS
     .        (Trigger_Sec, Data_Type, Source_Location, 
     .         N_Channels, Time_Range, Thresh, 
     .         Detector, History_Name, Energy_Calib_Scheme,
     .         lut_filename, lut_checksum, gain_cor,
     .         Object_Class, drmdbpath, atscat_path, calling_code,debug,Status, 
     .         Message)
C Changes CAWH 2008-Oct-10 - Added lut_filename, lut_checksum, gain_cor.
C Implemented Energy_Calib_Scheme. Added key words INFILE01, GBMCKSUM,
C GAIN_COR and CH2E_VER to report these values in the fits header of the
C EBOUNDS extension.    
C Changes CAWH 2008-March-06 - added drmdbpath and atscat_path as input variables
C  added keywords DIRDRMDB and DIRSCTDB to report these variables in the fits
C file header for audit trail.  
      implicit none
      ! COMMON /VERSION/ MNEMONIC, VERSION, FILE_VER
      CHARACTER*5    FILE_VER /'1.0.0'/ 
      CHARACTER*5    VERSION

      !                             INPUT
      ! The time (MET seconds) relative to which times will be stored in the
      ! FITS file (usually the BATSE burst start time, BSTST)
      REAL*8         Trigger_Sec
      ! Range of energies in which we are to compute a rate history
      CHARACTER*6    Data_Type
      REAL           Source_Location(2)
      INTEGER        N_Channels
      ! The time interval (seconds relative to the burst start time, BSTST)
      ! in which data is to be copied to the PDB
      REAL*8         Time_Range(2)
      REAL           Thresh(129)
      ! Number of the detector from which data is to be taken
      INTEGER        Detector
      ! Name of the file to be created, channel to energy conversion
      ! scheme used, name of file containing lut used to construct
      ! EBOUNDS array, and LUT checksum, and calling code if any
      CHARACTER*(*) History_Name, Energy_Calib_Scheme, lut_filename, 
     . lut_checksum, calling_code 
      REAL          gain_cor
      ! Flags which data records must satisfy
      CHARACTER*20   Object_Class
      CHARACTER*80  drmdbpath, atscat_path ! paths to drm and atscat DB
      Integer        debug !be verbose if >1
      !                            OUTPUT
      INTEGER        Status  ! Zero if successful
      CHARACTER*80   Message ! Error message if Status NE 0

      !                            EXTERNAL
      INTEGER        String_Length ! FUNCTION
      INTEGER        Index
      !                            EXTERNAL
      REAL*8         SEC_TO_DATE   !Function
      REAL*8         Fracday

      !                            LOCAL
      CHARACTER*3    Detector_Type
      CHARACTER*6    Detector_ID
      INTEGER        I,J  ! Generic indices
      INTEGER        TJD
      INTEGER        MONTH, DAY, YEAR, HH, MM
      REAL*8         SS
      REAL*8         StartTime, EndTime
      CHARACTER*20   Start_str, End_str
      REAL           Pi /3.141592654/
      ! FITSIO stuff
      INTEGER        FITS_Unit
      INTEGER        Blocksize, hdutype
      REAL*8         Year_Day
      REAL*8         Seconds
      CHARACTER*20   TType(10)
      CHARACTER*20   TForm(10)
      CHARACTER*20   TUnit(10)
      INTEGER        NAxes
      INTEGER*2      Channels(130)
      INTEGER        NAxis
      INTEGER        BitPix
      INTEGER        PCount
      INTEGER        GCount
      LOGICAL        Simple
      LOGICAL        Extend
      CHARACTER      C
      INTEGER        nullj
      INTEGER        testlen

      ! Lower thresholds (keV) of each energy channel in a spectrum
      REAL           Lo_enrg, Hi_enrg
      INTEGER        Burst_Day
      REAL*8         Burst_Sec
      CHARACTER*6    Date_Str
      INTEGER*2        DRM_Num
C cawh 7/24/2007 check inputs
      if (debug.gt.1) then
        print*,trigger_sec,data_type,
     .   source_location(1),source_location(2),
     .   n_channels,time_range(1),time_range(2),
     .   detector,history_name,
     .   energy_calib_scheme,object_class,status,message
        print*,'thresh=',(thresh(i),i=1,n_channels+1)
      endif	
C  *****************************
C  *  Executable code begins   *
C  *****************************
      status = 0
      nullj=0
      call ftgiou(FITS_unit,status)

      Message = 'FITS file construction error.'
c      FITS_Unit = 69
      Blocksize = 0 !2880
      CALL FTInit(FITS_Unit, History_Name, Blocksize, Status)
      IF (Status .NE. 0) THEN
         CALL FTGERR(Status, Message)
         RETURN
         END IF

c      NAxes(1) = 0
c      NAxes(2) = 0
      NAxis = 0
      BitPix = 8
      PCount = 0
      GCount = 1
      Simple = .TRUE.
      Extend = .TRUE.
      CALL FTPHPr(Fits_Unit, Simple, BitPix, NAxis, NAxes, PCount, 
     .     GCount, Extend, Status)
      IF (Status .NE. 0) THEN
         CALL FTGERR(Status, Message)
         RETURN
         END IF
      VERSION = '2.0'    ! v1.1 8/14/07 RDP: Several changes to fix ICD issues.
                         ! v1.2 3/5/08 CAWH: Changes to audit trail drm and scat db
                         ! v1.3 (see gbmrsp.f)
			 ! v1.4 (see gbmrsp.f)
			 ! v1.5 (see gbmrsp.f)
			 ! v1.6 (see gbmrsp.f)
			 ! v1.7 (see gbmrsp.f)
			 ! v1.7t (different version of
			 ! highres_ephoton_interpolator-extra error trap, also
			 !uses ehcan_integrator.REV.f 
			 ! v1.8pr - pre-release (see gbmrsp.f) - added integration over photon
			 ! side of matrix after call to echan_integrator.
			 ! - in make_DRM_FITS - added calling code as input and
			 !  added to CREATOR string. Changed CREATOR string from GBMRespGen to GBMRSP.
			 ! v1.8rc - release candidate (see gbmrsp.f) - fixed bug in SEC_TO_DATE.f
			 !  - changed limits on # of photon edges back to 200.
      			 ! v1.8 - released version -same as v1.8rc
			 ! v1.81 - released version - small bug fix to make
			 ! read_one_drm mode work.
			 ! v2.0t - changes to allowed number of photon energies.
			 ! v2.0 - released version (same as 2.0t). Now allows up to 2999 photon bins.
      testlen=string_length(calling_code)
      j=index(calling_code,' ')
c      print*,'Testlen = ',testlen,' first space index: ',j
c      print*,'Calling code = ',calling_code
      if (testlen.gt.0) then
        CALL FTPkyS(FITS_Unit, 'CREATOR', calling_code(1:j)//' GBMRSP V' // VERSION, 
     .     'Program creating this file', Status)
      else 
        CALL FTPkyS(FITS_Unit, 'CREATOR', ' GBMRSP V' // VERSION, 
     .     'Program creating this file', Status)
      endif
      CALL FTPkyS(FITS_Unit, 'FILETYPE', 'GBM DRM', ' ', 
     .     Status)
      CALL FTPkyS(FITS_Unit, 'FILE-VER', FILE_VER, 
     .     'Version of FITS file format', Status)
      CALL FTPDat(FITS_Unit, Status)
      I = String_Length(History_Name)
      J = INDEX(History_Name, '/') + 1
      CALL FTPkyS(FITS_Unit, 'FILENAME',  History_Name(J:I),    
     .     'Name of FITS file', Status)
      StartTime = Time_Range(1) + DBLE(86400.d0 * 51910.d0)
      if (debug.gt.1) print*,'starttime: ',starttime
      Fracday = SEC_TO_DATE(StartTime, MONTH, DAY, YEAR, HH, MM, SS)
      if (debug.gt.1) print*,month,day,year,hh,mm,ss
      CALL FTTm2s(YEAR, MONTH, DAY, HH, MM, SS,0, Start_str, Status)
      if (debug.gt.1) print*,start_str
      CALL FTPkyS(FITS_Unit, 'DATE-OBS',  Start_str,     
     .     'YYYY.DDD at start of data', Status)
      EndTime = Time_Range(2) + DBLE(86400.d0 * 51910.d0)
      if (debug.gt.1) print*,'start/end times: ',starttime,endtime
      day=0
      fracday = SEC_TO_DATE(EndTime, MONTH, DAY, YEAR, HH, MM, SS)
      if (debug.gt.1) print*,month,day,year,hh,mm,ss
      CALL FTTm2s(YEAR, MONTH, DAY, HH, MM, SS,0, End_str, Status)
      if (debug.gt.1) print*,'End str: ',end_str
      CALL FTPkyS(FITS_Unit, 'DATE-END',   End_str,     
     .     'YYYY.DDD at end of data', Status)
      CALL FTPkyG(FITS_Unit, 'MJDREFI',   51910.D0, 0,     
     .     'MJD date of reference epoch, integer part', Status)
      CALL FTPkyD(FITS_Unit, 'MJDREFF',   7.428703703703703D-4, 15, 
     .     'MJD date of reference epoch, fractional part', Status)
      CALL FTPkyD(FITS_Unit, 'TSTART', Time_Range(1), 12, 
     .     'Observation start time relative to MJDREF', Status)
      CALL FTPkyD(FITS_Unit, 'TSTOP',  Time_Range(2),   12, 
     .     'Observation stop time relative to MJDREF', Status)
      CALL FTPkyG(FITS_Unit, 'TRIGTIME', Trigger_Sec, 6, 
     .     'Trigger time (s) relative to MJDREF', Status)
      CALL FTPkyS(FITS_Unit, 'TIMESYS',   'TT',   
     .     'Time system used in time keywords', Status)
      CALL FTPkyS(FITS_Unit, 'TIMEUNIT',   's',   
     .     'Time unit used in TSTART, TSTOP', Status)
      CALL FTPkyS(FITS_Unit, 'TELESCOP', 'GLAST',    
     .     'Name of mission / spacecraft', Status)
      CALL FTPkyS(FITS_Unit, 'INSTRUME', 'GBM', 
     .     'Name of instrument generating data', Status)

      IF (Detector .LT. 10) THEN
         Detector_Type = 'NAI'
         Lo_enrg = 0.
         Hi_enrg = 2000.
         WRITE (Detector_ID, 300) Detector
300      FORMAT ('NAI_0', I1)
      ELSE IF ((detector.GE.10).and.(detector.lt.12)) THEN
         Detector_Type = 'NAI'
         Lo_enrg = 0.
         Hi_enrg = 2000.
         WRITE (Detector_ID, 301) Detector
301      FORMAT ('NAI_', I2)
      ELSE
         Detector_Type = 'BGO'
         Lo_enrg = 0.
         Hi_enrg = 50000.
         WRITE (Detector_ID, 400) Detector - 12
400      FORMAT ('BGO_0', I1)
      END IF
      if (debug.gt.1) print*,'Detector_ID = ',Detector_ID

      CALL FTPkyS(FITS_Unit, 'DETNAM', Detector_ID, 
     .     'Individual detector name', Status)
      CALL FTPkyS(FITS_Unit, 'OBSERVER', 'Meegan',       
     .     'Principal investigator for GBM', Status)
      CALL FTPkyS(FITS_Unit, 'ORIGIN',   'GIOC',   
     .     'Name of organization making file', Status)
      CALL FTPkyS(FITS_Unit, 'OBJECT', Object_Class, 
     .     'Object Designation (GRB for bursts, etc.)', Status)
      CALL FTPkyS(FITS_Unit, 'RADECSYS',   'FK5',   
     .     'Stellar reference frame', Status)
      CALL FTPkyF(FITS_Unit, 'EQUINOX',  2000.0, 1,            
     .     'Equinox for RA and Dec', Status)
      CALL FTPkyF(FITS_Unit, 'RA_OBJ',  Source_Location(1), 1,
     .     'J2000 RA of source, degrees', Status)
      CALL FTPkyF(FITS_Unit, 'DEC_OBJ', Source_Location(2), 1,
     .     'J2000 DEC of source, degrees', Status)
      CALL FTPkyJ(FITS_Unit, 'DRM_NUM', nullj, 
     .     'Number of DRMs stored in this file', Status)
      CALL FTPkyS(FITS_Unit, 'DRM_TYPE', Data_Type, 
     .     'Data type for which DRM is intended', Status)
      CALL FTPkyS(FITS_Unit, 'DIRDRMDB', drmdbpath, 
     .     'Path to direct detector response database', Status)
      CALL FTPkyS(FITS_Unit, 'DIRSCTDB', atscat_path, 
     .     'Path to atmospheric scattering database', Status)
      CALL FTPCOM(FITS_Unit, 
     .     'This file contains GBM time-sequenced ' // 
     .     'detector response matrices', Status)
C      CALL FTPkyS(FITS_Unit, 'PRIMTYPE', 'NONE',  
C     .     'No primary array', Status)
      CALL FTPDef(FITS_Unit, 8, 0, 0, 0, 1, Status)
      IF (Status .NE. 0) THEN
         CALL FTGERR(Status, Message)
         RETURN
         END IF
C compute and write the DATASUM and CHECKSUM keywords to the primary header
       CALL ftpcks(FITS_Unit, status)       
      ! *** Create a header for the energy calibration extension. ***
C  Move to the last (1st) HDU in the file .
      status=0
      call ftmahd(FITS_unit,1,hdutype,status)

      CALL FTCrHd(FITS_Unit, Status)
      IF (Status .NE. 0) THEN
         CALL FTGERR(Status, Message)
         RETURN
         END IF

      TType(1) = 'CHANNEL'
      TType(2) = 'E_MIN'
      TType(3) = 'E_MAX'

      TForm(1) = '1I'
      TForm(2) = '1E'
      TForm(3) = '1E'

      TUnit(1) = ' '
      TUnit(2) = 'keV'
      TUnit(3) = 'keV'

      CALL FTPHBn(FITS_Unit, N_Channels, 3, TType, TForm, TUnit, 
     .           'EBOUNDS', 0, Status)
      IF (Status .NE. 0) THEN
         CALL FTGERR(Status, Message)
         RETURN
         END IF

      CALL FTMCOM(FITS_Unit, 'TTYPE1',   
     .     'Detector channel number', Status)
      CALL FTIkyJ(FITS_Unit, 'TLMIN1', 0, 
     .     'Channel numbers are non-negative', Status)
      CALL FTIkyJ(FITS_Unit, 'TLMAX1', 130, 
     .     'More than the number of channels', Status)
      CALL FTMCOM(FITS_Unit, 'TTYPE2', 
     .     'Low energy bound for channel', Status)
      CALL FTIkyE(FITS_Unit, 'TLMIN2', Lo_enrg, 3, 
     .     'Lowest channel energy', Status)
      CALL FTIkyE(FITS_Unit, 'TLMAX2', Hi_enrg, 3, 
     .     'Highest channel energy', Status)
      CALL FTMCOM(FITS_Unit, 'TTYPE3', 
     .     'High energy bound for channel', Status)
      CALL FTIkyE(FITS_Unit, 'TLMIN3', Lo_enrg, 3, 
     .     'Lowest channel energy', Status)
      CALL FTIkyE(FITS_Unit, 'TLMAX3', Hi_enrg, 3, 
     .     'Highest channel energy', Status)

      CALL FTPkyJ(FITS_Unit, 'EXTVER', 1, 
     .     'Version of this extension format', Status)
      CALL FTPDat(FITS_Unit, Status)

      CALL FTPkyS(FITS_Unit, 'DATE-OBS',  Start_str,
     .     'YYYY.DDD at start of data', Status)
      CALL FTPkyS(FITS_Unit, 'DATE-END',   End_str,     
     .     'YYYY.DDD at end of data', Status)
      CALL FTPkyG(FITS_Unit, 'MJDREFI',   51910.d0, 0, 
     .     'MJD date of reference epoch, integer part', Status)
      CALL FTPkyD(FITS_Unit, 'MJDREFF',   7.428703703703703D-4, 15, 
     .     'MJD date of reference epoch, fractional part', Status)
      CALL FTPkyD(FITS_Unit, 'TSTART', Time_Range(1), 12, 
     .     'Observation start time relative to MJDREF', Status)
      CALL FTPkyD(FITS_Unit, 'TSTOP',  Time_Range(2),   12, 
     .     'Observation stop time relative to MJDREF', Status)
      CALL FTPkyG(FITS_Unit, 'TRIGTIME', Trigger_Sec, 6, 
     .     'Trigger time (s) relative to MJDREF', Status)
      CALL FTPkyS(FITS_Unit, 'TIMESYS',   'TT',   
     .     'Time system used in time keywords', Status)
      CALL FTPkyS(FITS_Unit, 'TIMEUNIT',   's',   
     .     'Time unit used in TSTART, TSTOP', Status)
      CALL FTPkyS(FITS_Unit, 'TELESCOP', 'GLAST',    
     .     'Name of mission / spacecraft', Status)
      CALL FTPkyS(FITS_Unit, 'INSTRUME', 'GBM', 
     .     'Name of instrument generating data', Status)
      CALL FTPkyS(FITS_Unit, 'DETNAM', Detector_ID, 
     .     'Individual detector name', Status)
      CALL FTPkyS(FITS_Unit, 'OBSERVER', 'Meegan',       
     .     'Principal investigator for GBM', Status)
      CALL FTPkyS(FITS_Unit, 'ORIGIN',   'GIOC',   
     .     'Name of organization making file', Status)
      CALL FTPkyS(FITS_Unit, 'OBJECT',  Object_Class, 
     .     'Object Designation (GRB for bursts, etc.)', Status)
      CALL FTPkyS(FITS_Unit, 'RADECSYS',   'FK5',   
     .     'Stellar reference frame', Status)
      CALL FTPkyF(FITS_Unit, 'EQUINOX',  2000.0, 1,           
     .     'Equinox for RA and Dec', Status)
      CALL FTPkyF(FITS_Unit, 'RA_OBJ',  Source_Location(1), 1,
     .     'J2000 RA of source, degrees', Status)
      CALL FTPkyF(FITS_Unit, 'DEC_OBJ', Source_Location(2), 1,
     .     'J2000 DEC of source, degrees', Status)

      CALL FTPkyS(FITS_Unit, 'FILTER',   'none',   
     .     'Not applicable to GBM', Status)
      CALL FTPkyS(FITS_Unit, 'CHANTYPE',  'PHA',   
     .     'No corrections have been applied', Status)
      CALL FTPkyJ(FITS_Unit, 'DETCHANS', N_Channels, 
     .     'Total number of channels in each rate', Status)
C New key words added CAWH 10-22-2008 in response to CCR 99 to 
C provide audit trail info for channel to energy look-up table (LUT)     
      CALL FTPkyS(FITS_Unit, 'INFILE01',lut_filename,
     .     'The LUT used in the construction of the EBOUNDS array',
     .     status) 
      CALL FTPkyS(FITS_Unit, 'GBMCKSUM',lut_checksum,
     .     'GBM FSW icrc checksum value (hex)',status)
      CALL FTPkyS(FITS_Unit, 'CH2E_VER',Energy_Calib_Scheme,
     .     'Channel to energy conversion scheme used',status)
      CALL FTPkyF(FITS_Unit, 'GAIN_COR',gain_cor, 8,
     .     'Gain correction factor applied to energy edges',status) 
      
      CALL FTPkyS(FITS_Unit, 'HDUCLASS',  'OGIP',   '', Status)
      CALL FTPkyS(FITS_Unit, 'HDUVERS',   '1.2.0',  '', Status)
      CALL FTPkyS(FITS_Unit, 'HDUCLAS1',  'RESPONSE', 
     .     'Typically found in RMF files', Status)
      CALL FTPkyS(FITS_Unit, 'HDUCLAS2',  'EBOUNDS',  
     .     'From CAL/GEN/92-002', Status)

      ! *** Write the data table for the energy calibration extension. ***

      CALL FTBDef(FITS_Unit, 3, TForm, 0, N_Channels, Status)
      IF (Status .NE. 0) THEN
         CALL FTGERR(Status, Message)
         RETURN
         END IF

      ! One row for each channel.
      DO I = 1, N_Channels
          Channels(I) = I - 1
          END DO
	  CALL FTPCLI(FITS_Unit, 1, 1, 1, N_Channels, Channels, Status)
      DO I = 1, N_Channels

          CALL FTPCLE(FITS_Unit, 2, I, 1, 1, Thresh(I), Status)

          CALL FTPCLE(FITS_Unit, 3, I, 1, 1, Thresh(I + 1), Status)
          END DO

C compute and write the DATASUM and CHECKSUM keywords to the primary header
      CALL ftpcks(FITS_Unit, status)       

      CALL FTClos(FITS_Unit, Status)
      CALL FTfiou(FITS_unit, status)
      IF (Status .NE. 0) THEN
         CALL FTGERR(Status, Message)
         END IF

      END ! SUBROUTINE Make_Spectral_History
C**********************************************************************
C
C
C**********************************************************************
      SUBROUTINE Make_Response
     .        (Trigger_Sec, Source_Location, Source_AZ_El, 
     .         Earth_Location, N_Channels, Num_Ebins, Time_Range, 
     .         EThresh, Detector, filename, matrix,maxen,maxchan,
     .         Object_Class, drmfiles, atscat_file, matrix_type,
     .         debug, Status, Message)
C changes CAWH 2008-April-08 changed dimension of Ethresh from 129 to maxen to
C  match gbmrsp.f
C changes CAWH 2008-March-06 Added input variables: drmfiles, atscat_file,
C  and matrix_type
C added header keywords INFILE01-INFILE04. INFILE01-INFILE03 = drmfiles, 
C unless matrix_type = 1 (scattering only). Then INFILE01=atscat_file. 
C If matrix_type = 2, then INFILE04 = atscat_file.     
      implicit none
      ! COMMON /VERSION/ MNEMONIC, VERSION, FILE_VER
      CHARACTER*5    FILE_VER /'1.0.0'/
      CHARACTER*5    VERSION /'1.2'/

      !                             INPUT
      ! The time (TJD seconds) relative to which times will be stored in the
      ! FITS file (usually the BATSE burst start time, BSTST)
      REAL*8         Trigger_Sec
      REAL           Source_Location(2), Source_AZ_El(2)
      REAL           Earth_Location(2)
      !Angles between the detector normal and the source and geocenter
      REAL           det_ang, geo_ang
      INTEGER        N_Channels, Num_Ebins,maxen,maxchan
      ! The time interval (seconds relative to the burst start time, BSTST)
      ! in which data is to be copied to the PDB
      REAL*8         Time_Range(2)
      ! Range of energies in which we are to compute a rate history
      REAL           EThresh(num_ebins+1)
      ! Number of the detector from which data is to be taken
      INTEGER        Detector
      ! Name of the file to be created:
      CHARACTER*(*)  filename
      ! Flags which data records must satisfy
      CHARACTER*20   Object_Class
      ! Uncompressed DRM num_ebins X n_channels
      REAL           matrix(maxen, maxchan)
      ! List of direct detector response input datafiles
      CHARACTER*40   drmfiles(3)
      !ATSCAT_FILE = Name of file containing atmospheric scattering db
      CHARACTER*40   atscat_file
      !Matrix type: 0=direct, 1=scattered, 2=direct+scattered.    
      INTEGER        matrix_type
      integer        debug !increase verbosity if >1
      !                            OUTPUT
      INTEGER        Status  ! Zero if successful
      CHARACTER*80   Message ! Error message if Status NE 0

      !                            EXTERNAL
      INTEGER        String_Length ! FUNCTION
      !                            EXTERNAL
      REAL*8         SEC_TO_DATE   !Function
      REAL*8         Fracday

      !                            LOCAL
      CHARACTER*3    Detector_Type
      CHARACTER*6    Detector_ID
      INTEGER        I, J  ! Generic indices
      INTEGER        TJD, non_zeros, first_chan
      INTEGER        MONTH, DAY, YEAR, HH, MM
      REAL*8	     SS
      REAL*8         StartTime, EndTime
      CHARACTER*20   Start_str, End_str
      CHARACTER*6    Date_Str
      INTEGER        nrows
      INTEGER        unit,nelem
      INTEGER        DRM_Num
      ! FITSIO stuff
      INTEGER        FITS_Unit
      INTEGER        Blocksize, readwrite, hdutype, tfields, nrows
     .               nelem, colnum, frow, felem, varidat,nullj
      CHARACTER*20   TType(10)
      CHARACTER*20   TForm(10)
      CHARACTER*20   TUnit(10)
      character*16   extname
      LOGICAL        anynull
      character*34   drm_comment
      real           values(n_channels)
      if (debug.gt.1) then
        print*,Trigger_Sec, Source_Location, Source_AZ_El, 
     .         Earth_Location, N_Channels, Num_Ebins, Time_Range, 
     .         EThresh, Detector, filename, 
     .         Object_Class, Status
        print*,'inside make response. Matrix = '
        do i = 1,num_ebins
          print*,i,(matrix(i,j),j=1,n_channels)
        enddo	
      endif	
C  *****************************
C  *  Executable code begins   *
C  *****************************
      nullj=0
      status=0
      Message = 'FITS file construction error.'
C      FITS_Unit = 69
      Blocksize = 2880
C      CALL FTInit(FITS_Unit, filename, Blocksize, Status)
C      IF (Status .NE. 0) THEN
C         CALL FTGERR(Status, Message)
C         RETURN
C         END IF

C  Get an unused Logical Unit Number to use to open the FITS file.
C  This routine is not required;  programmers can choose any unused
C  unit number to open the file.
      call ftgiou(FITS_Unit,status)
C  Open the FITS file, with read/write access.
      readwrite=1
      call ftopen(FITS_Unit,filename,readwrite,blocksize,status)
C  Get the number of DRM extensions currently in the file:
      CALL FTGkyJ(FITS_Unit, 'DRM_NUM', DRM_Num, drm_comment, Status)
C  We are adding a new MATRIX extension (always assuming success!)
      DRM_Num = DRM_Num + 1
C  Now update the number of DRM extensions in the file before we move on:
      CALL FTUkyJ(FITS_Unit, 'DRM_NUM', DRM_Num, drm_comment, Status)
C  Update the endtime of the observation to reflect the additional matrix:
      EndTime = Time_Range(2) + DBLE(86400.d0 * 51910.d0)
      fracday= SEC_TO_DATE(EndTime, MONTH, DAY, YEAR, HH, MM, SS)
      if (debug.gt.1) 
     . print*,'MONTH, DAY, YEAR, HH, MM, SS:',MONTH, DAY, YEAR, HH, MM, SS
      CALL FTTm2s(YEAR, MONTH, DAY, HH, MM, SS,0, End_str, Status)
      if (debug.gt.1) then
        print*,'End_str = ',end_str
        print*,'status = ',status
      endif	
      CALL FTUkyS(FITS_Unit, 'DATE-END', End_str, 
     .   'YYYY.DDD at end of data', Status)
      CALL FTUkyD(FITS_Unit, 'TSTOP',    Time_Range(2),       12, 
     .   'Observation stop time relative to MJDREF', Status)
C compute and write the DATASUM and CHECKSUM keywords to the primary header
      CALL ftpcks(FITS_Unit, status)       
c      CALL FTClos(FITS_Unit, Status)
c      print*,'status = ',status
c      CALL FTfiou(FITS_unit, status)
c      print*,'status = ',status
c      stop
C  Move to the next (2nd) HDU in the file and update.
      call ftmahd(FITS_Unit,2,hdutype,status)
      CALL FTUkyS(FITS_Unit, 'DATE-END', End_str, 
     .   'YYYY.DDD at end of data', Status)
      CALL FTUkyD(FITS_Unit, 'TSTOP',    Time_Range(2),       12, 
     .   'Observation stop time relative to MJDREF', Status)
C compute and write the DATASUM and CHECKSUM keywords to the primary header
      CALL ftpcks(FITS_Unit, status)       
C  Append/create a new empty HDU onto the end of the file and move to it.
      call ftcrhd(FITS_Unit,status)
      
      tfields = 6
      nrows = Num_Ebins
      extname = 'SPECRESP MATRIX'
      varidat=0

      TType(1) = 'ENERG_LO'
      TType(2) = 'ENERG_HI'
      TType(3) = 'N_GRP'
      TType(4) = 'F_CHAN'
      TType(5) = 'N_CHAN'
      TType(6) = 'MATRIX'

      TForm(1) = '1E'
      TForm(2) = '1E'
      TForm(3) = '1I'
      TForm(4) = 'PI(128)'
      TForm(5) = 'PI(128)'
      TForm(6) = 'PE(128)'

      TUnit(1) = 'keV'
      TUnit(2) = 'keV'
      TUnit(3) = ' '
      TUnit(4) = ' '
      TUnit(5) = ' '
      TUnit(6) = 'cm**2'

C  FTPHBN writes all the required header keywords which define the
C  structure of the binary table. NROWS and TFIELDS gives the number of
C  rows and columns in the table, and the TTYPE, TFORM, and TUNIT arrays
C  give the column name, format, and units, respectively of each column.
      if (debug.gt.1) print*,'Calling ftphbn:'
      call ftphbn(FITS_unit,nrows,tfields,ttype,tform,tunit,
     &            extname,varidat,status)
      IF (Status .NE. 0) THEN
         CALL FTGERR(Status, Message)
	 stop
         RETURN
         END IF
      frow=1
      felem=1
      colnum=1
      
      CALL FTMCOM(FITS_Unit, 'TTYPE1',   
     .     'Lower energy bound of the energy bin', Status)
      CALL FTMCOM(FITS_Unit, 'TTYPE2',   
     .     'Upper energy bound of the energy bin', Status)
      CALL FTMCOM(FITS_Unit, 'TTYPE3',   
     .     'The number of channel subsets in this row', Status)
      CALL FTMCOM(FITS_Unit, 'TTYPE4',   
     .     'First channel number of each channel subset', Status)
      CALL FTMCOM(FITS_Unit, 'TTYPE5',   
     .     'The number of channels in each channel subset', Status)
      CALL FTMCOM(FITS_Unit, 'TTYPE6',   
     .     'The compressed detector response matrix', Status)
      StartTime = Time_Range(1) + DBLE(86400.d0 * 51910.d0)
      if (debug.gt.1) print*,Starttime
      fracday = SEC_TO_DATE(StartTime, MONTH, DAY, YEAR, HH, MM, SS)
      if (debug.gt.1) print*,Month,day,year,hh,mm,ss
      CALL FTTm2s(YEAR, MONTH, DAY, HH, MM, SS,0, Start_str, Status)
      if (debug.gt.1) then
        print*,'Start_str = ',start_str
        print*,'status = ',status
      endif	
      CALL FTPkyJ(FITS_Unit, 'EXTVER', DRM_Num, 
     .     'Version of this extension format', Status)
      CALL FTPDat(FITS_Unit, Status)
      CALL FTPkyS(FITS_Unit, 'DATE-OBS',  Start_str,     
     .     'YYYY.DDD at start of data', Status)
      CALL FTPkyS(FITS_Unit, 'DATE-END',   End_str,     
     .     'YYYY.DDD at end of data', Status)
      CALL FTPkyG(FITS_Unit, 'MJDREFI',   51910.D0, 0, 
     .     'MJD date of reference epoch, integer part', Status)
      CALL FTPkyD(FITS_Unit, 'MJDREFF',   7.428703703703703D-4, 15, 
     .     'MJD date of reference epoch, fractional part', Status)
      CALL FTPkyD(FITS_Unit, 'TSTART', Time_Range(1), 12, 
     .     'Observation start time relative to MJDREF', Status)
      CALL FTPkyD(FITS_Unit, 'TSTOP',  Time_Range(2),   12, 
     .     'Observation stop time relative to MJDREF', Status)
      CALL FTPkyG(FITS_Unit, 'TRIGTIME', Trigger_Sec, 6, 
     .     'Trigger time (s) relative to MJDREF', Status)
      CALL FTPkyS(FITS_Unit, 'TIMESYS',   'TT',   
     .     'Time system used in time keywords', Status)
      CALL FTPkyS(FITS_Unit, 'TIMEUNIT',   's',   
     .     'Time unit used in TSTART, TSTOP', Status)
      CALL FTPkyS(FITS_Unit, 'TELESCOP', 'GLAST',    
     .     'Name of mission / spacecraft', Status)
      CALL FTPkyS(FITS_Unit, 'INSTRUME', 'GBM', 
     .     'Name of instrument generating data', Status)
      IF (Detector .LT. 10) THEN
         Detector_Type = 'NAI'
         WRITE (Detector_ID, 300) Detector
300      FORMAT ('NAI_0', I1)
      ELSE IF ((detector.GE.10).and.(detector.lt.12)) THEN
         Detector_Type = 'NAI'
         WRITE (Detector_ID, 301) Detector
301      FORMAT ('NAI_', I2)
      ELSE
         Detector_Type = 'BGO'
         WRITE (Detector_ID, 400) Detector - 12
400      FORMAT ('BGO_0', I1)
      END IF
      if (debug.gt.1) print*,'Detector_ID = ',Detector_ID
      CALL FTPkyS(FITS_Unit, 'DETNAM', Detector_ID, 
     .     'Individual detector name', Status)
      if(matrix_type.eq.0) then
        CALL FTPkyS(FITS_Unit, 'MAT_TYPE', 'Direct',       
     .     'Response Matrix Type', Status)
      else if(matrix_type.eq.1) then
        CALL FTPkyS(FITS_Unit, 'MAT_TYPE', 'Scattered',       
     .     'Response Matrix Type', Status)      
      else if(matrix_type.eq.2) then
        CALL FTPkyS(FITS_Unit, 'MAT_TYPE', 'Summed',       
     .     'Response Matrix Type', Status)      
      endif  
      if (debug.gt.1)
     . print*,'status = ',status,' detector_id = ',detector_id
      CALL FTPkyS(FITS_Unit, 'OBSERVER', 'Meegan',       
     .     'Principal investigator for GBM', Status)
      CALL FTPkyS(FITS_Unit, 'ORIGIN',   'GIOC',   
     .     'Name of organization making file', Status)
      CALL FTPkyJ(FITS_Unit, 'RSP_NUM', DRM_Num, 
     .     'Response matrix index number', Status)
      CALL FTPkyS(FITS_Unit, 'OBJECT',  Object_Class, 
     .     'Object Designation (GRB for bursts, etc.)', Status)
      CALL FTPkyS(FITS_Unit, 'RADECSYS',   'FK5',   
     .     'Stellar reference frame', Status)
      CALL FTPkyF(FITS_Unit, 'EQUINOX',  2000.0, 1,            
     .     'Equinox for RA and Dec', Status)
      CALL FTPkyF(FITS_Unit, 'RA_OBJ',  Source_Location(1), 1,
     .     'J2000 RA of source, degrees', Status)
      CALL FTPkyF(FITS_Unit, 'DEC_OBJ', Source_Location(2), 1,
     .     'J2000 DEC of source, degrees', Status)
      CALL FTPkyF(FITS_Unit, 'SRC_AZ',  Source_AZ_El(1), 1, 
     .     'Azimuth of source in spacecraft coordinates', Status)
      CALL FTPkyF(FITS_Unit, 'SRC_EL', Source_AZ_El(2), 1,
     .     'Elevation of source in spacecraft coordinates', Status)
      CALL FTPkyF(FITS_Unit, 'GEO_AZ',  Earth_Location(1), 1,
     .     'Azimuth of geocenter in spacecraft coordinates', Status)
      CALL FTPkyF(FITS_Unit, 'GEO_EL', Earth_Location(2), 1, 
     .     'Elevation of geocenter in spacecraft coordinates', Status)
      CALL detector_source_offset (source_az_el(1),source_az_el(2),
     . detector, det_ang)
       CALL detector_source_offset (Earth_location(1),Earth_Location(2),
     . detector, geo_ang)
    
      if (detector.le.11) then
        CALL FTPkyF(FITS_Unit, 'DET_ANG',  det_ang, 1, 
     .     'Angle between source and detector normal', Status) 
        CALL FTPkyF(FITS_Unit, 'GEO_ANG',  geo_ang, 1, 
     .     'Angle between geocenter and detector normal', Status) 
      elseif (detector.eq.12) then
        CALL FTPkyF(FITS_Unit, 'DET_ANG',  det_ang, 1, 
     .     'Angle between source and spacecraft +X-axis', Status) 
        CALL FTPkyF(FITS_Unit, 'GEO_ANG',  geo_ang, 1, 
     .     'Angle between geocenter and spacecraft +X-axis', Status) 
      elseif (detector.eq.13) then 
        CALL FTPkyF(FITS_Unit, 'DET_ANG',  det_ang, 1, 
     .     'Angle between source and spacecraft -X-axis', Status) 
        CALL FTPkyF(FITS_Unit, 'GEO_ANG',  geo_ang, 1, 
     .     'Angle between geocenter and spacecraft -X-axis', Status) 
      endif
        
      CALL FTPkyS(FITS_Unit, 'FILTER',   'none',   
     .     'Not applicable to GBM', Status)
      CALL FTPkyS(FITS_Unit, 'CHANTYPE',  'PHA',   
     .     'No corrections have been applied', Status)
      CALL FTPkyJ(FITS_Unit, 'NUMEBINS', Num_Ebins, 
     .     'Number of true energy bins of the MATRIX', Status)
      CALL FTPkyJ(FITS_Unit, 'DETCHANS', N_Channels, 
     .     'Total number of channels in each rate', Status)
      if((matrix_type.eq.0).or.(matrix_type.eq.2)) then
        CALL FTPkyS(FITS_Unit, 'INFILE01',   drmfiles(1),   
     .     'Detector response database input file 1', Status)
        CALL FTPkyS(FITS_Unit, 'INFILE02',   drmfiles(2),   
     .     'Detector response database input file 2', Status)
        CALL FTPkyS(FITS_Unit, 'INFILE03',   drmfiles(3),   
     .     'Detector response database input file 3', Status)
      else if(matrix_type.eq.1) then
        CALL FTPkyS(FITS_Unit, 'INFILE01',   atscat_file,   
     .     'Atmospheric scattering database input file', Status)
      endif
      if(matrix_type.eq.2) then
        CALL FTPkyS(FITS_Unit, 'INFILE04',   atscat_file,   
     .     'Atmospheric scattering database input file', Status)
      endif 
      
       
      CALL FTPkyS(FITS_Unit, 'HDUCLASS',  'OGIP',   '', Status)
      CALL FTPkyS(FITS_Unit, 'HDUVERS',   '1.3.0',  '', Status)
      CALL FTPkyS(FITS_Unit, 'HDUCLAS1',  'RESPONSE', 
     .     'Typically found in RMF files', Status)
      CALL FTPkyS(FITS_Unit, 'HDUCLAS2',  'RSP_MATRIX',  
     .     'From CAL/GEN/92-002', Status)

      do i=1,Num_Ebins
        do j=1,n_channels
          values(j) = 0.
        enddo	
          call ftpcle(FITS_Unit,1,i,felem,1,EThresh(i),status) 
          call ftpcle(FITS_Unit,2,i,felem,1,EThresh(i+1),status) 
          nelem = 0
          non_zeros = N_Channels
          first_chan = 1
C          found = 0
          do while ((matrix(i,first_chan) .EQ. 0.0) .AND. 
     .              (first_chan .LT. N_Channels))
              first_chan = first_chan + 1
              non_zeros = non_zeros - 1
C              found = 1
          enddo
	  do j=1, non_zeros
	    values(j) = matrix(i,first_chan+j-1)
	  enddo  
c	  if (debug.gt.1) print*,'i = ',i,'values = ',values
C          if (found .EQ. 0) j = 0
C  The number of groups for our matrices will always be one (even though there 
C  may be a gap corresponding to the K-edge or 511 line). 
c  It's simpler this way....
          call ftpclj(FITS_Unit,3,i,felem,1,1,status) 
          call ftpclj(FITS_Unit,4,i,felem,1,first_chan,status) 
          call ftpclj(FITS_Unit,5,i,felem,1,non_zeros,status) 
c          do j=1,non_zeros
c              call ftpcle(FITS_Unit,6,i,j,1,matrix(i, first_chan+j-1),
c     .                    status) 
c          enddo
          call ftpcle(FITS_UNIT,6,i,1,non_zeros,
     .          values,status)
      enddo	 

C compute and write the DATASUM and CHECKSUM keywords to the extension header
      CALL ftpcks(FITS_Unit, status)       

      CALL FTClos(FITS_Unit, Status)
      CALL FTfiou(FITS_unit, status)

      IF (Status .NE. 0) THEN
         CALL FTGERR(Status, Message)
         END IF

      END ! SUBROUTINE Make_Response
